-module(ea12srv).
-export([start/0, stop/0, dump/0]).
-export([connect/2, disconnect/1, send/2]).

% -------------------- Interface functions ---------------------------
start() ->
    register(chatsrv, spawn(fun init/0)).

stop() ->
    chatsrv ! stop.

dump() ->
    chatsrv ! {dump, self()},
    receive Users -> Users end.

% -------------------- Client interface -----------------------------
connect(Srv, Nick) ->
    % a chatsrv regisztrlt nv az Srv node-on
    {chatsrv, Srv} ! {connect, self(), Nick}.

disconnect(Srv) ->
    {chatsrv, Srv} ! {disconnect, self()}.

send(Srv, Text) ->
    {chatsrv, Srv} ! {text, self(), Text}.

% -------------------- Implementation functions ----------------------
init() ->
    process_flag(trap_exit, true),
    loop([]).

loop(Users) ->
    receive
        stop ->
            ok;

        {dump, Pid} ->
            Pid ! Users,
            loop(Users);
        
        {connect, Pid, Nick} ->
            link(Pid),
            loop([ {Nick, Pid} | Users ]);

        {disconnect, Pid} ->
            loop(remove(Users, Pid));

        {text, _From, Text} ->
            lists:foreach(fun({_,Pid}) -> ea12cli:send(Pid, Text) end, Users),
            loop(Users);

        {'EXIT', From, _} ->
            loop(remove(Users, From));

        Any ->
            io:format("[~w]~n", [Any]),
            loop(Users)
    end.

remove(Users, Pid) ->
    lists:filter(fun ({_, P}) -> P /= Pid end, Users).
