-module(ea13srv).
-export([start/1, stop/0, dump/0]).
-export([connect/2, disconnect/1, send/2]).

-define(CLIENT,ea13cli).

% -------------------- Interface functions ---------------------------
start(Max) ->
    register(chatsrv, spawn(fun() -> init(Max) end)).

stop() ->
    chatsrv ! stop.

dump() ->
    chatsrv ! {dump, self()},
    receive Users -> Users end.

% -------------------- Client interface -----------------------------
connect(Srv, Nick) ->
    % a chatsrv regisztrlt nv az Srv node-on
    {chatsrv, Srv} ! {connect, self(), Nick},
    receive
        ok -> ok;
        deny -> deny
        after 1000 -> timeout
    end.

disconnect(Srv) ->
    {chatsrv, Srv} ! {disconnect, self()}.

send(Srv, Text) ->
    {chatsrv, Srv} ! {text, self(), Text}.

% -------------------- Implementation functions ----------------------
-record(state, {users, max}).

init(Max) ->
    process_flag(trap_exit, true),
    loop(#state{users=[], max=Max}).

loop(State = #state{users = Users}) ->
    receive
        stop ->
            ok;

        {dump, Pid} ->
            Pid ! Users,
            loop(State);
        
        {connect, Pid, Nick} ->
            if
                length(Users) >= State#state.max ->
                    Pid ! deny,
                    loop(State);
                true ->
                    link(Pid),
                    Pid ! ok,
                    loop(State#state{ users = [ {Nick, Pid} | Users ]})
            end;

        {text, From, Text} ->
            [{Nick, _}] = lists:filter(fun({_,Pid}) -> Pid == From end, Users),
            lists:foreach(
                fun({_,Pid}) ->
                    ?CLIENT:send(Pid, Nick ++ ": " ++ Text)
                end, Users),
            loop(State);

        {disconnect, Pid} ->
            loop(State#state{ users = remove(Users, Pid)});

        {'EXIT', From, _} ->
            loop(State#state{ users = remove(Users, From)});

        Any ->
            io:format("[~w]~n", [Any]),
            loop(State)
    end.

remove(Users, Pid) ->
    lists:filter(fun ({_, P}) -> P /= Pid end, Users).
