-module(ea14srv).
-export([start/1, stop/0, dump/0]).
-export([connect/2, disconnect/1, send/2, join/2]).

-define(CLIENT,ea14cli).

% -------------------- Interface functions ---------------------------
start(Max) ->
    register(chatsrv, spawn(fun() -> init(Max) end)).

stop() ->
    chatsrv ! stop.

dump() ->
    chatsrv ! {dump, self()},
    receive Users -> Users end.

% -------------------- Client interface -----------------------------
connect(Srv, Nick) ->
    % a chatsrv regisztrlt nv az Srv node-on
    {chatsrv, Srv} ! {connect, self(), Nick},
    receive
        ok -> ok;
        deny -> deny
        after 1000 -> timeout
    end.

disconnect(Srv) ->
    {chatsrv, Srv} ! {disconnect, self()}.

send(Srv, Text) ->
    {chatsrv, Srv} ! {text, self(), Text}.

join(Srv, Group) ->
    {chatsrv, Srv} ! {join, self(), Group}.

% -------------------- Implementation functions ----------------------
-record(state, {users, max}).
-record(user, {nick, pid, group = "Global"}).

init(Max) ->
    process_flag(trap_exit, true),
    loop(#state{users=ets:new(users, [{keypos, #user.pid}]), max=Max}).

loop(State = #state{users = Users}) ->
    receive
        stop ->
            ok;

        {dump, Pid} ->
            Pid ! ets:match(Users, #user{nick='$1', group='$2', _='_'}),
            loop(State);
        
        {connect, Pid, Nick} ->
            if
                length(Users) >= State#state.max ->
                    Pid ! deny,
                    loop(State);
                true ->
                    link(Pid),
                    Pid ! ok,
                    ets:insert(Users, #user{nick=Nick, pid=Pid}),
                    loop(State)
            end;

        {text, From, Text} ->
            [#user{nick=Nick, group=Group}] = ets:lookup(Users, From),
            lists:foreach(
                fun([Pid]) ->
                    ?CLIENT:send(Pid, Nick ++ ": " ++ Text)
                end,
                ets:match(Users, #user{group=Group,pid='$1',_='_'})),
            loop(State);

        {join, Pid, Group} ->
            [User] = ets:lookup(Users, Pid),
            ets:insert(Users, User#user{group = Group}),
            loop(State);

        {disconnect, Pid} ->
            ets:delete(Users, Pid),
            loop(State);

        {'EXIT', From, _} ->
            ets:delete(Users, From),
            loop(State);

        Any ->
            io:format("[~w]~n", [Any]),
            loop(State)
    end.

