-module(cli).
-export([start/1, send/2]).

-define(SERVER,srv).

start(Nick) ->
    case ?SERVER:connect(Nick) of
        deny ->
            io:format("Connection failed~n",[]);
        timeout ->
            io:format("No reply~n",[]);
        ok ->
            Loop = self(),
            spawn(fun() -> input(Loop) end),
            loop()
    end.

loop() ->
    receive
        quit ->
            ?SERVER:disconnect();

        {send, Text} ->
            ?SERVER:send(Text),
            loop();
        
        {text, Text} ->
            io:format("~s~n", [Text]),
            loop();

        {join, Group} ->
            ?SERVER:join(Group),
            loop()
    end.

send(Pid, Text) ->
    Pid ! {text, Text}.

input(Loop) ->
    case string:strip(io:get_line('--> '), right, $\n) of
        "#q" ->
            Loop ! quit,
            ok;
        "#j " ++ Group ->
            Loop ! {join, Group},
            input(Loop);
        S ->
            Loop ! {send, S},
            input(Loop)
    end.
