-module(grp_srv).
-behaviour(gen_server).

-export([start_link/2, text/3, stop/1]).
-export([init/1, handle_cast/2, handle_call/3, handle_info/2,
        terminate/2, code_change/3]).

-include("srv.hrl").
-record(state, {users, name}).

start_link(Users, Name) ->
    gen_server:start_link(?MODULE, {Users, Name}, []).

text(Srv, Nick, Text) -> 
    gen_server:cast(Srv, {text, Nick, Text}).

stop(Srv) ->
    gen_server:cast(Srv, {stop}).

init({Users, Name}) ->
    {ok, #state{users=Users, name=Name}}.

handle_cast({text, Nick, Text}, S=#state{users=Users, name=Name}) ->
    lists:foreach(
        fun([Pid]) ->
            ?CLIENT:send(Pid, Nick ++ ": " ++ Text)
        end,
        ets:match(Users, #user{group=Name, pid='$1',_='_'})),
    {noreply, S};

handle_cast({stop}, S) ->
    {stop, S}.

handle_call(_, _From, _S) ->
    exit(function_clause).

handle_info(_, S) ->
    {noreply, S}.

terminate(_, _S) ->
    ok.

code_change(_, S, _) ->
    {ok, S}.
